***
*** Determine Who Has A File Open Using NovLib
***
*** Syntax: WHO [FileName | UserName | ConnectionNumber]
***
***
*** by: Eric E Sorenson
***

#include "DIRECTRY.CH"
#include "NOVLIB.CH"

#xcommand DEFAULT <var> TO <InitVal> => ;
          IF <var> == NIL ; <var> := <InitVal> ; ENDIF


*
* Entry Function
*
* Parameters : IN  - sFileName - Char - Opt - FileName/User Name/Connection
*              OUT - None
*
function Who(sFileName)
local sFullPath, aFileList

SET CENTURY ON
*
* Fill In Optional Command Line Parameters
*
DEFAULT sFileName TO "*.*"
sFileName := upper(sFileName)
*
* Determine What Type Of WHO Request Has Been Made
*
do case
   case (val(sFileName) > 0) .and. (val(sFileName) <= 256)
      *
      * Files Open For A Connection Number
      *
      ShowConnFiles(val(sFileName))
   case (Conns(sFileName) > 0 ) 
      * slight fix by Goth to work around bug in NetWare NETX.VLM V1.10
      *
      * Files For A User
      *
      ShowUserFiles(sFileName)
   otherwise
      *
      * Users With A File Open
      *
      ShowFileUsers(sFileName)
endcase

return (NIL)


*
* Show All Of The Users With A File Open
*
static function ShowFileUsers( sFileName )
local sFullPath, aFileList, sVolume, cDrive
*
* Get The Full Novell Path To Use For The File(s)
*
sFullPath := "" ; cDrive := CurrDrive()
if empty(at(":",sFileName))
   sVolume := GetNetPath(DirHandle())
   sFullPath := left(sVolume,at(":",sVolume)) + sFullPath
endif
if (at(":",sFileName) == 2)
   cDrive := left(sFileName,1) ; sFileName := substr(sFileName,3)
   sVolume := GetNetPath(DirHandle(cDrive))
   sFullPath := left(sVolume,at(":",sVolume)) + sFullPath
endif
if empty(at("\",sFileName))
   sFullPath += upper(CurDir(cDrive)) + "\"
endif
if (right(sFileName,1) $ ":\") .or. empty(sFileName)
   sFileName += "*.*"
endif
if IsDir(sFileName)
   sFileName += "\*.*"
endif
if (left(sFileName,1) == "\")
   sFileName := substr(sFileName,2)
endif
sFileName := upper(sFullPath + sFileName)
sFullPath := left(sFileName, rat("\",sFileName)-1)
*
* Get The List Of Files Matching This File Spec
*
aFileList := directory(sFileName)
asort(aFileList,,, { |a,b| a[F_NAME] < b[F_NAME] } )
*
* Print Headings
*
? "Users who currently have",sFileName,"open:"
?
? "File Name     Connection  User Name        Login Time           Full Name      "
? "        "
*
* Print The List For Each File
*
aeval(aFileList, { |f| ShowOpenList(sFullPath,f[F_NAME]) })
*
* Done
*
?

return (NIL)


*
* Show The List Of Users Who Have A File Open
*
static function ShowOpenList(sFullPath, sFileName)
local nCnt, nStaNo, sFileSpec := sFullPath+"\"+sFileName
*
* Get The First Station That Has The File Open
*
nCnt := 1
nStaNo := WhoHasFile( sFileSpec, nCnt )
if (NWError() == -255)
   *
   * File Is Opened Exclusively
   *
   ? padr(sFileName,12)+"* "+padc(str(nStaNo,3),10)+MyConChar(nStaNo)+" "+padr(LoginName(nStaNo),15)+"  "+dtoc(LoginDate(nStaNo))+" "+LoginTime(nStaNo)+"  " +left(UserFName(LoginName(nStaNo)),15)
else
   *
   * Show All Connections With The File Open
   *
   while (nStaNo <> -1)
      ? padr(sFileName,12)+"  "+padc(str(nStaNo,3),10)+MyConChar(nStaNo)+" "+padr(LoginName(nStaNo),15)+"  "+dtoc(LoginDate(nStaNo))+" "+LoginTime(nStaNo)+"  " +left(UserFName(LoginName(nStaNo)),15)
      nCnt++
      nStaNo := WhoHasFile( sFileSpec, nCnt )
   enddo
endif

return (NIL)


*
* Format The Connection Number, Flagging The User's Connection
*
static function MyConChar( nConNo )
static nMyConNo
local sRet

if (nMyConNo == NIL)
   nMyConNo := ConnNo()
endif

sRet := iif(nConNo == nMyConNo,"*"," ")

return (sRet)


*
* Display The List Of Files That A User Has Open
*
static function ShowUserFiles( sUserName )
local nConNo, aConnList, i, nCnt, sFileName, sLoginTime
*
* Get The List Of Connections That This User Is Logged In At
*
aConnList := {}
nCnt := Conns(sUserName)
for i = 1 to nCnt
   nConNo := Conns()
   if (nConNo >= 0)
      aadd(aConnList, nConNo)
   endif
next i
*
* Print Headings
*
? "Files currently open by user " + sUserName + ":"
?
? "Connection  Login Time          File Name                                      "
? "----------  ------------------- -----------------------------------------------"
*
* Show The List Of Files For Each Connection
*
for i := 1 to len(aConnList)
   nCnt := 1
   nConNo := aConnList[i]
   sLoginTime := dtoc(LoginDate(nConNo))+" "+LoginTime(nConNo)
   sFileName := WhoHasFile(nConNo, nCnt )
   while !empty( sFileName )
      if (len(sFileName) > 47)
         sFileName := "..." + right(sFileName,44)
      endif
      ? padc(str(nConNo,3),10) + MyConChar(nConNo) + " " + sLoginTime + " " + sFileName
      sFileName := WhoHasFile( nConNo, ++nCnt )
   enddo
next i
*
* Done
*
?

return (NIL)


*
* Display The List Of Files That A Connection Has Open
*
static function ShowConnFiles( nConNo )
local nCnt, sFileName, sLoginTime
*
* Print Headings
*
? "Files currently open by connection " + alltrim(str(nConNo)) + "/" + LoginName(nConNo) + ":"
?
? "Connection  Login Time          File Name                                      "
? "   "
nCnt := 1
sLoginTime := dtoc(LoginDate(nConNo))+" "+LoginTime(nConNo)
*
* Print The List Of Files
*
sFileName := WhoHasFile(nConNo, nCnt )
while !empty( sFileName )
   if (len(sFileName) > 47)
      sFileName := "..." + right(sFileName,44)
   endif
   ? padc(str(nConNo,3),10) + MyConChar(nConNo) + " " + sLoginTime + " " + sFileName
   sFileName := WhoHasFile( nConNo, ++nCnt )
enddo
*
* Done
*
?

return (NIL)


*
* Test For The Existance Of A Directory Path
*
* Parameters : IN  - sPath - Char - Req - Path To Test For
*              OUT - lRet  - Log        - Does This Directory Exist ?
*
function IsDir( sPath )
local lRet, sTest, nPos
*
* Remove Any File Name Or Wildcard From The Path
*
sTest := sPath
if ("*" $ sTest) .or. ("." $ sTest)
   nPos := rat("\")
   if empty(nPos)
      nPos := rat(":")
   endif
   if empty(nPos)
      sTest := ""
   else
      sTest := left(sTest,nPos-1)
   endif
endif
if empty(sTest)
   lRet := .F.
else
   lRet := (len(directory( sTest, "D" )) > 0)
endif

return (lRet)
